#ifndef HW_PUB_H
#define HW_PUB_H
#include <linux/wait.h>
#include <linux/list.h>
#include <linux/types.h>
#define NOT_SUPPORT	-2
#define NO_CONFIG	-1
#define NOT_READY	1

#define ZRHUNG_CMD_INVALID (0xFF)

#define CLIENT_NAME_LEN						(32) /* max client name length */
#define DSM_MAX_DEVICE_NAME_LEN				(32) /* max device name length */
#define DSM_MAX_MODULE_NAME_LEN				(16)  /* max module name length */
#define DSM_MAX_IC_NAME_LEN					(16)  /* max ic name length */

typedef enum {
    ZRHUNG_CAT_SYS,
    ZRHUNG_CAT_APPEYE,
    ZRHUNG_CAT_EVENT,
    ZRHUNG_CAT_NUM_MAX,
    //CATs below dont have config
    ZRHUNG_CAT_XCOLLIE
} zrhung_cat_type;

struct dsm_dev{
	const char *name;
	const char *device_name;
	const char *ic_name;
	const char *module_name;
	struct dsm_client_ops *fops;
	size_t buff_size;
};

struct dsm_client{
	struct list_head dsm_client_entry;
	char client_name[CLIENT_NAME_LEN];
	char device_name[DSM_MAX_DEVICE_NAME_LEN];
	char ic_name[DSM_MAX_IC_NAME_LEN];
	char module_name[DSM_MAX_MODULE_NAME_LEN];
	struct dsm_dev* dev;
	struct dsm_client* realclient;
	bool dsm_init_ok;
	int client_id;
	int error_no;
	unsigned long buff_flag;
	struct dsm_client_ops *cops;
	wait_queue_head_t waitq;
	size_t read_size;
	size_t used_size;
	size_t buff_size;
	u8 dump_buff[];
};

typedef enum {
    //system Zero Hung watchpoint
    ZRHUNG_WP_NONE = ZRHUNG_CAT_SYS << 8 | 0x0000,
    ZRHUNG_WP_HUNGTASK,
    ZRHUNG_WP_SCREENON,
    ZRHUNG_WP_SCREENOFF,
    ZRHUNG_WP_SR,
    ZRHUNG_WP_LMKD,
    ZRHUNG_WP_IO,
    ZRHUNG_WP_CPU,
    ZRHUNG_WP_GC,
    ZRHUNG_WP_TP,
    ZRHUNG_WP_FENCE,
    ZRHUNG_WP_SCRNONFWK,
    ZRHUNG_WP_SERVICE,
    ZRHUNG_WP_GPU,
    ZRHUNG_WP_TRANS_WIN,
    ZRHUNG_WP_FOCUS_WIN,
    ZRHUNG_WP_LCD,
    ZRHUNG_WP_RES_LEAK,
    ZRHUNG_WP_IPC_THREAD,
    ZRHUNG_WP_IPC_OBJECT,
    ZRHUNG_WP_UFS,
    ZRHUNG_WP_INIT,
    ZRHUNG_WP_VMWATCHDOG,
    ZRHUNG_WP_APPFREEZE,
    ZRHUNG_WP_APPFRZ_WARNING,
    ZRHUNG_WP_HTSK_WARNING,
    ZRHUNG_WP_FDLEAK,
    ZRHUNG_WP_CAMERA,
    ZRHUNG_WP_UPLOAD_BIGDATA,
    ZRHUNG_WP_TEMP1,
    ZRHUNG_WP_TEMP2,
    ZRHUNG_WP_TEMP3,
    ZRHUNG_WP_MAX,

    //application eye
    APPEYE_NONE = ZRHUNG_CAT_APPEYE << 8 | 0x0000,
    APPEYE_UIP_WARNING,
    APPEYE_UIP_FREEZE,
    APPEYE_UIP_SLOW,
    APPEYE_UIP_INPUT,
    APPEYE_MTO_FREEZE,
    APPEYE_MTO_SLOW,
    APPEYE_MTO_INPUT,
    APPEYE_BF,
    APPEYE_CL,
    APPEYE_CLA,
    APPEYE_ARN,
    APPEYE_MANR,
    APPEYE_CANR,
    APPEYE_FWB_WARNING,
    APPEYE_FWB_FREEZE,
    APPEYE_NFW,
    APPEYE_TWIN,
    APPEYE_FWE,
    APPEYE_SBF,
    APPEYE_RCV,
    APPEYE_OBS,
    APPEYE_RECOVER_RESULT,
    APPEYE_MT,
    APPEYE_UIP_RECOVER,
    APPEYE_BINDER_TIMEOUT,
    APPEYE_SBF_FREEZE,
    APPEYE_SBF_RECOVER,
    APPEYE_SUIP_WARNING,
    APPEYE_SUIP_FREEZE,
    APPEYE_SUIP_INPUT,
    APPEYE_CRASH,
    APPEYE_TEMP1,
    APPEYE_TEMP2,
    APPEYE_TEMP3,

    APPEYE_MAX,

    //system event watchpoint
    ZRHUNG_EVENT_NONE = ZRHUNG_CAT_EVENT << 8 | 0x0000,
    ZRHUNG_EVENT_LONGPRESS,
    ZRHUNG_EVENT_POWERKEY,
    ZRHUNG_EVENT_HOMEKEY,
    ZRHUNG_EVENT_BACKKEY,
    ZRHUNG_EVENT_MULTIPRESS,
    ZRHUNG_EVENT_TEMP1,
    ZRHUNG_EVENT_TEMP2,
    ZRHUNG_EVENT_TEMP3,

    ZRHUNG_XCOLLIE_NONE = ZRHUNG_CAT_XCOLLIE << 8 | 0x0000,
    XCOLLIE_FWK_SERVICE,

    ZRHUNG_XCOLLIE_MAX,

    ZRHUNG_EVENT_MAX
} zrhung_wp_id;

#ifdef CONFIG_HW_PROXY
struct dsm_client *dsm_register_client (struct dsm_dev *dev);
void dsm_unregister_client (struct dsm_client *dsm_client,struct dsm_dev *dev);
struct dsm_client *dsm_find_client(char *dsm_name);
int dsm_client_ocuppy(struct dsm_client *client);
int dsm_client_unocuppy(struct dsm_client *client);
int dsm_client_record(struct dsm_client *client, const char *fmt, ...);
int dsm_client_copy(struct dsm_client *client, void *src, int sz);
#ifdef CONFIG_HUAWEI_DATA_ACQUISITION
int dsm_client_copy_ext(struct dsm_client *client, void *src, int sz);
#endif
void dsm_client_notify(struct dsm_client *client, int error_no);
int dsm_update_client_vendor_info(struct dsm_dev *dev);
long zrhung_ioctl(struct file *file, unsigned int cmd, unsigned long arg);
int zrhung_send_event(zrhung_wp_id id, const char* cmd_buf, const char* msg_buf);
int zrhung_get_config(zrhung_wp_id wp, char *data, uint32_t maxlen);
#endif
#endif
